#
#  ANC350lib is a Python implementation of the C++ header provided
#     with the attocube ANC350 closed-loop positioner system.
#
#  It depends on anc350v4.dll and libusb0.dll, which are provided by attocube in the
#     ANC350_Library folder on the driver disc. Place all
#     of these in the same folder as this module (and that of ANC350lib).
#     This should also work with anc350v3.dll, although this has not been thoroughly checked.
#
#                ANC350lib is written by Rob Heath
#                      rob@robheath.me.uk
#                         24-Feb-2015
#                       robheath.me.uk
#
#                 ANC350v4lib by Brian Schaefer
#                      bts72@cornell.edu
#                         5-Jul-2016
#              http://nowack.lassp.cornell.edu/


import ctypes, os, time

#
# List of error types
#
ANC_Ok = 0 #                    No error
ANC_Error = -1 #                Unknown / other error
ANC_Timeout = 1 #               Timeout during data retrieval
ANC_NotConnected = 2 #          No contact with the positioner via USB
ANC_DriverError = 3 #           Error in the driver response
ANC_DeviceLocked = 7 #          A connection attempt failed because the device is already in use
ANC_Unknown = 8 # Unknown error.
ANC_NoDevice = 9 # Invalid device number used in call
ANC_NoAxis = 10 # Invalid axis number in function call
ANC_OutOfRange = 11 # Parameter in call is out of range
ANC_NotAvailable = 12 # Function not available for device type

#checks the errors returned from the dll
def checkError(code,func,args):
    if code == ANC_Ok:
        return
    elif code == ANC_Error:             
        raise Exception("Error: unspecific in"+str(func.__name__)+"with parameters:"+str(args))
    elif code == ANC_Timeout:           
        raise Exception("Error: comm. timeout in"+str(func.__name__)+"with parameters:"+str(args))
    elif code == ANC_NotConnected:      
        raise Exception("Error: not connected") 
    elif code == ANC_DriverError:       
        raise Exception("Error: driver error") 
    elif code == ANC_DeviceLocked:      
        raise Exception("Error: device locked")
    elif code == ANC_NoDevice:
        raise Exception("Error: invalid device number")
    elif code == ANC_NoAxis:
        raise Exception("Error: invalid axis number")
    elif code == ANC_OutOfRange:
        raise Exception("Error: parameter out of range")
    elif code == ANC_NotAvailable:
        raise Exception("Error: function not available")
    else:                    
        raise Exception("Error: unknown in"+str(func.__name__)+"with parameters:"+str(args))
    return code

# import dll - have to change directories so it finds libusb0.dll
'''
directory_of_this_module_and_dlls = os.path.dirname(os.path.realpath(__file__))
current_directory = os.getcwd()
os.chdir(directory_of_this_module_and_dlls)
anc350v4 = ctypes.windll.LoadLibrary(directory_of_this_module_and_dlls+'\\anc350v4.dll')
os.chdir(current_directory)
'''
anc350v4 = ctypes.windll.anc350v4

#aliases for the strangely-named functions from the dll
discover = getattr(anc350v4,"ANC_discover")
getDeviceInfo = getattr(anc350v4,"ANC_getDeviceInfo")
connect = getattr(anc350v4,"ANC_connect")
disconnect = getattr(anc350v4,"ANC_disconnect")
getDeviceConfig = getattr(anc350v4,"ANC_getDeviceConfig")
getAxisStatus = getattr(anc350v4,"ANC_getAxisStatus")
setAxisOutput = getattr(anc350v4,"ANC_setAxisOutput")
setAmplitude = getattr(anc350v4,"ANC_setAmplitude")
setFrequency = getattr(anc350v4,"ANC_setFrequency")
setDcVoltage = getattr(anc350v4,"ANC_setDcVoltage")
getAmplitude = getattr(anc350v4,"ANC_getAmplitude")
getFrequency = getattr(anc350v4,"ANC_getFrequency")
startSingleStep = getattr(anc350v4,"ANC_startSingleStep")
startContinousMove = getattr(anc350v4,"ANC_startContinousMove")
startAutoMove = getattr(anc350v4,"ANC_startAutoMove")
setTargetPosition = getattr(anc350v4,"ANC_setTargetPosition")
setTargetRange = getattr(anc350v4,"ANC_setTargetRange")
getPosition = getattr(anc350v4,"ANC_getPosition")
getFirmwareVersion = getattr(anc350v4,"ANC_getFirmwareVersion")
configureExtTrigger = getattr(anc350v4,"ANC_configureExtTrigger")
configureAQuadBIn = getattr(anc350v4,"ANC_configureAQuadBIn")
configureAQuadBOut = getattr(anc350v4,"ANC_configureAQuadBOut")
configureRngTriggerPol = getattr(anc350v4,"ANC_configureRngTriggerPol")
configureRngTrigger = getattr(anc350v4,"ANC_configureRngTrigger")
configureRngTriggerEps = getattr(anc350v4,"ANC_configureRngTriggerEps")
configureNslTrigger = getattr(anc350v4,"ANC_configureNslTrigger")
configureNslTriggerAxis = getattr(anc350v4,"ANC_configureNslTriggerAxis")
selectActuator = getattr(anc350v4,"ANC_selectActuator")
getActuatorName = getattr(anc350v4,"ANC_getActuatorName")
getActuatorType = getattr(anc350v4,"ANC_getActuatorType")
measureCapacitance = getattr(anc350v4,"ANC_measureCapacitance")
saveParams = getattr(anc350v4,"ANC_saveParams")

#set error checking & handling
discover.errcheck = checkError
connect.errcheck = checkError
disconnect.errcheck = checkError
getDeviceConfig.errcheck = checkError
getAxisStatus.errcheck = checkError
setAxisOutput.errcheck = checkError
setAmplitude.errcheck = checkError
setFrequency.errcheck = checkError
setDcVoltage.errcheck = checkError
getAmplitude.errcheck = checkError
getFrequency.errcheck = checkError
startSingleStep.errcheck = checkError
startContinousMove.errcheck = checkError
startAutoMove.errcheck = checkError
setTargetPosition.errcheck = checkError
setTargetRange.errcheck = checkError
getPosition.errcheck = checkError
getFirmwareVersion.errcheck = checkError
configureExtTrigger.errcheck = checkError
configureAQuadBIn.errcheck = checkError
configureAQuadBOut.errcheck = checkError
configureRngTriggerPol.errcheck = checkError
configureRngTrigger.errcheck = checkError
configureRngTriggerEps.errcheck = checkError
configureNslTrigger.errcheck = checkError
configureNslTriggerAxis.errcheck = checkError
selectActuator.errcheck = checkError
getActuatorName.errcheck = checkError
getActuatorType.errcheck = checkError
measureCapacitance.errcheck = checkError
saveParams.errcheck = checkError



import logging

logger = logging.getLogger(__name__)

try:
	import pyanc350.v2
	logger.info('found anc350v2.dll')
except OSError:
	logger.warning('could not find anc350v2.dll')

try:
	import pyanc350.v3
	logger.info('found anc350v3.dll')
except OSError:
	logger.warning('could not find anc350v3.dll')

try:
	import pyanc350.v4
	logger.info('found anc350v4.dll')
except OSError:
	logger.warning('could not find anc350v4.dll')


#
#  PyANC350v4 is a control scheme suitable for the Python coding style
#    for the attocube ANC350 closed-loop positioner system.
#
#  It implements ANC350v4lib, which in turn depends on anc350v4.dll and libusb0.dll, which are provided by attocube in the
#     ANC350_Library folder on the driver disc. Place all
#     of these in the same folder as this module (and that of ANC350lib).
#     This should also work with anc350v3.dll, although this has not been thoroughly checked.
#
#  Unlike ANC350v4lib which is effectively a re-imagining of the
#    C++ header, PyANC350v4 is intended to behave as one might expect
#    Python to. This means: returning values; behaving as an object.
#
#  At present this only addresses the first ANC350 connected to the
#    machine.
#
#  Usage:
#  1. instantiate Positioner() class to begin, eg. pos = Positioner().
#  2. methods from the ANC350v2 documentation are implemented such that
#      function ANC_getPosition(device, axisNo, &position)
#      becomes position = pos.getPosition(axisNo), for example. Return code handling is
#      within ANC350lib.
#  3. for tidiness remember to Positioner.close() when finished!
#
#                PyANC350 is written by Rob Heath
#                      rob@robheath.me.uk
#                         24-Feb-2015
#                       robheath.me.uk
#
#                 PyANC350v4 by Brian Schaefer
#                      bts72@cornell.edu
#                         5-Jul-2016
#              http://nowack.lassp.cornell.edu/

import pyanc350.v3.ANC350libv3 as ANC
import ctypes, math

class Positioner:
    
    def __init__(self):
        self.discover()
        self.device = self.connect()
        
        
    def configureAQuadBIn(self, axisNo, enable, resolution):
        '''
        Enables and configures the A-Quad-B (quadrature) input for the target position.
        Parameters
            axisNo	Axis number (0 ... 2)
            enable	Enable (1) or disable (0) A-Quad-B input
            resolution	A-Quad-B step width in m. Internal resolution is 1 nm.
        Returns
            None
        '''
        ANC.configureAQuadBIn(self.device, axisNo, enable, ctypes.c_double(resolution))
        
        
    def configureAQuadBOut(self, axisNo, enable, resolution, clock):
        '''
        Enables and configures the A-Quad-B output of the current position.

        Parameters
            axisNo	Axis number (0 ... 2)
            enable	Enable (1) or disable (0) A-Quad-B output
            resolution	A-Quad-B step width in m; internal resolution is 1 nm
            clock	Clock of the A-Quad-B output [s]. Allowed range is 40ns ... 1.3ms; internal resulution is 20ns.
        Returns/
            None
        '''
        ANC.configureAQuadBOut(self.device, axisNo, enable, ctypes.c_double(resolution), ctypes.c_double(clock))
       
    
    def configureExtTrigger(self, axisNo, mode):
        '''
        Enables the input trigger for steps.

        Parameters
            axisNo	Axis number (0 ... 2)
            mode	Disable (0), Quadratur (1), Trigger(2) for external triggering
        Returns
            None
        '''
        ANC.configureExtTrigger(self.device, axisNo, mode)
     
    
    def configureNslTrigger(self, enable):
        '''
        Enables NSL Input as Trigger Source.

        Parameters
            enable	disable(0), enable(1)
        Returns
            None
        '''
        ANC.configureNslTrigger(self.device, enable)
    
    
    def configureNslTriggerAxis(self, axisNo):
        '''
        Selects Axis for NSL Trigger.

        Parameters
            axisNo	Axis number (0 ... 2)
        Returns
            None
        '''
        ANC.configureNslTriggerAxis(self.device, axisNo)
      
    
    def configureRngTrigger(self, axisNo, lower, upper):
        '''
        Configure lower position for range Trigger.

        Parameters
            axisNo	Axis number (0 ... 2)
            lower	Lower position for range trigger (nm)
            upper	Upper position for range trigger (nm)
        Returns
            None
        '''
        ANC.configureRngTrigger(self.device, axisNo, lower, upper)
       
    
    def configureRngTriggerEps(self, axisNo, epsilon):
        '''
        Configure hysteresis for range Trigger.

        Parameters
            axisNo	Axis number (0 ... 2)
            epsilon	hysteresis in nm / mdeg
        Returns
            None
        '''
        ANC.configureRngTriggerEps(self.device, axisNo, epsilon)
        
        
    def configureRngTriggerPol(self, axisNo, polarity):
        '''
        Configure lower position for range Trigger.

        Parameters
            axisNo	Axis number (0 ... 2)
            polarity	Polarity of trigger signal when position is between lower and upper Low(0) and High(1)
        Returns
            None
        '''
        ANC.configureRngTriggerPol(self.device, axisNo, polarity)
       
    
    def connect(self, devNo=0):
        '''
        Initializes and connects the selected device. This has to be done before any access to control variables or measured data.

        Parameters
            devNo	Sequence number of the device. Must be smaller than the devCount from the last ANC_discover call. Default: 0
        Returns
            device	Handle to the opened device, NULL on error
        '''
        device = ctypes.c_void_p()
        ANC.connect(devNo, ctypes.byref(device))
        return device
        
        
    def disconnect(self):
        '''
        Closes the connection to the device. The device handle becomes invalid.

        Parameters
            None
        Returns
            None
        '''
        ANC.disconnect(self.device)
       
    
    def discover(self, ifaces=3):
        '''
        The function searches for connected ANC350RES devices on USB and LAN and initializes internal data structures per device. Devices that are in use by another application or PC are not found. The function must be called before connecting to a device and must not be called as long as any devices are connected.

        The number of devices found is returned. In subsequent functions, devices are identified by a sequence number that must be less than the number returned.

        Parameters
            ifaces	Interfaces where devices are to be searched. {None: 0, USB: 1, ethernet: 2, all:3} Default: 3
        Returns
            devCount	number of devices found
        '''
        devCount = ctypes.c_int()
        ANC.discover(ifaces, ctypes.byref(devCount))
        return devCount.value
    
    
    def getActuatorName(self, axisNo):
        '''
        Get the name of the currently selected actuator

        Parameters
            axisNo	Axis number (0 ... 2)
        Returns
            name	Name of the actuator
        '''
        name = ctypes.create_string_buffer(20)
        ANC.getActuatorName(self.device, axisNo, ctypes.byref(name))
        return name.value.decode('utf-8')
        
        
    def getActuatorType(self, axisNo):
        '''
        Get the type of the currently selected actuator

        Parameters
            axisNo	Axis number (0 ... 2)
        Returns
            type_	Type of the actuator {0: linear, 1: goniometer, 2: rotator}
        '''
        type_ = ctypes.c_int()
        ANC.getActuatorType(self.device, axisNo, ctypes.byref(type_))
        return type_.value
       
        
    def getAmplitude(self, axisNo):
        '''
        Reads back the amplitude parameter of an axis.
        
        Parameters
            axisNo	Axis number (0 ... 2)
        Returns
            amplitude	Amplitude V
        '''
        amplitude = ctypes.c_double()
        ANC.getAmplitude(self.device, axisNo, ctypes.byref(amplitude))
        return amplitude.value
    
    
    def getAxisStatus(self, axisNo):
        '''
        Reads status information about an axis of the device.

        Parameters
            axisNo	Axis number (0 ... 2)
        Returns
            connected	Output: If the axis is connected to a sensor.
            enabled	Output: If the axis voltage output is enabled.
            moving	Output: If the axis is moving.
            target	Output: If the target is reached in automatic positioning
            eotFwd	Output: If end of travel detected in forward direction.
            eotBwd	Output: If end of travel detected in backward direction.
            error	Output: If the axis' sensor is in error state.
        '''
        connected = ctypes.c_int()
        enabled = ctypes.c_int()
        moving = ctypes.c_int()
        target = ctypes.c_int()
        eotFwd = ctypes.c_int()
        eotBwd = ctypes.c_int()
        error = ctypes.c_int()

        ANC.getAxisStatus(self.device, axisNo, ctypes.byref(connected), ctypes.byref(enabled), ctypes.byref(moving), ctypes.byref(target), ctypes.byref(eotFwd), ctypes.byref(eotBwd), ctypes.byref(error))
        return connected.value, enabled.value, moving.value, target.value, eotFwd.value, eotBwd.value, error.value
    
    
    def getDeviceConfig(self):
        '''
        Reads static device configuration data

        Parameters
            None
        Returns
            featureSync	"Sync": Ethernet enabled (1) or disabled (0)
            featureLockin	"Lockin": Low power loss measurement enabled (1) or disabled (0)
            featureDuty	"Duty": Duty cycle enabled (1) or disabled (0)
            featureApp	"App": Control by IOS app enabled (1) or disabled (0)
        '''
        features = ctypes.c_int()
        ANC.getDeviceConfig(self.device, features)
        
        featureSync = 0x01&features.value
        featureLockin = (0x02&features.value)/2
        featureDuty = (0x04&features.value)/4
        featureApp = (0x08&features.value)/8
        
        return featureSync, featureLockin, featureDuty, featureApp

    
    def getDeviceInfo(self, devNo=0):
        '''
        Returns available information about a device. The function can not be called before ANC_discover but the devices don't have to be connected . All Pointers to output parameters may be zero to ignore the respective value.

        Parameters
            devNo	Sequence number of the device. Must be smaller than the devCount from the last ANC_discover call. Default: 0
        Returns
            devType	Output: Type of the ANC350 device. {0: Anc350Res, 1:Anc350Num, 2:Anc350Fps, 3:Anc350None}
            id	Output: programmed hardware ID of the device
            serialNo	Output: The device's serial number. The string buffer should be NULL or at least 16 bytes long.
            address	Output: The device's interface address if applicable. Returns the IP address in dotted-decimal notation or the string "USB", respectively. The string buffer should be NULL or at least 16 bytes long.
            connected	Output: If the device is already connected
        '''
        devType = ctypes.c_int()
        id_ = ctypes.c_int()
        serialNo = ctypes.create_string_buffer(16) 
        address = ctypes.create_string_buffer(16) 
        connected = ctypes.c_int()

        ANC.getDeviceInfo(devNo, ctypes.byref(devType), ctypes.byref(id_), ctypes.byref(serialNo), ctypes.byref(address), ctypes.byref(connected))
        return devType.value, id_.value, serialNo.value.decode('utf-8'), address.value.decode('utf-8'), connected.value
    
    
    def getFirmwareVersion(self):
        '''
        Retrieves the version of currently loaded firmware.

        Parameters
            None
        Returns
            version	Output: Version number
        '''
        version = ctypes.c_int()
        ANC.getFirmwareVersion(self.device, ctypes.byref(version))
        return version.value
    
    
    def getFrequency(self, axisNo):
        '''
        Reads back the frequency parameter of an axis.

        Parameters
            axisNo	Axis number (0 ... 2)
        Returns
            frequency	Output: Frequency in Hz
        '''
        frequency = ctypes.c_double()
        ANC.getFrequency(self.device, axisNo, ctypes.byref(frequency))
        return frequency.value
    
    
    def getPosition(self, axisNo):
        '''
        Retrieves the current actuator position. For linear type actuators the position unit is m; for goniometers and rotators it is degree.

        Parameters
            axisNo	Axis number (0 ... 2)
        Returns
            position	Output: Current position [m] or [°]
        '''
        position = ctypes.c_double()
        ANC.getPosition(self.device, axisNo, ctypes.byref(position))
        return position.value
    
    
    def measureCapacitance(self, axisNo):
        '''
        Performs a measurement of the capacitance of the piezo motor and returns the result. If no motor is connected, the result will be 0. The function doesn't return before the measurement is complete; this will take a few seconds of time.

        Parameters
            axisNo	Axis number (0 ... 2)
        Returns
            cap	Output: Capacitance [F]
        '''
        cap = ctypes.c_double()
        ANC.measureCapacitance(self.device, axisNo, ctypes.byref(cap))
        return cap.value
   

    def saveParams(self):
        '''
        Saves parameters to persistent flash memory in the device. They will be present as defaults after the next power-on. The following parameters are affected: Amplitude, frequency, actuator selections as well as Trigger and quadrature settings.

        Parameters
            None
        Returns
            None
        '''
        ANC.saveParams(self.device)
    
    
    def selectActuator(self, axisNo, actuator):
        '''
        Selects the actuator to be used for the axis from actuator presets.

        Parameters
            axisNo	Axis number (0 ... 2)
            actuator	Actuator selection (0 ... 255)
                0: ANPg101res
                1: ANGt101res
                2: ANPx51res
                3: ANPx101res
                4: ANPx121res
                5: ANPx122res
                6: ANPz51res
                7: ANPz101res
                8: ANR50res
                9: ANR51res
                10: ANR101res
                11: Test
        Returns
            None
        '''
        ANC.selectActuator(self.device, axisNo, actuator)
    
    
    def setAmplitude(self, axisNo, amplitude):
        '''
        Sets the amplitude parameter for an axis

        Parameters
            axisNo	Axis number (0 ... 2)
            amplitude	Amplitude in V, internal resolution is 1 mV
        Returns
            None
        '''
        ANC.setAmplitude(self.device, axisNo, ctypes.c_double(amplitude))
   

    def setAxisOutput(self, axisNo, enable, autoDisable):
        '''
        Enables or disables the voltage output of an axis.

        Parameters
            axisNo	Axis number (0 ... 2)
            enable	Enables (1) or disables (0) the voltage output.
            autoDisable	If the voltage output is to be deactivated automatically when end of travel is detected.
        Returns
            None
        '''
        ANC.setAxisOutput(self.device, axisNo, enable, autoDisable)
   

    def setDcVoltage(self, axisNo, voltage):
        '''
        Sets the DC level on the voltage output when no sawtooth based motion is active.

            Parameters
            axisNo	Axis number (0 ... 2)
            voltage	DC output voltage [V], internal resolution is 1 mV
        Returns
            None        
        '''
        ANC.setDcVoltage(self.device, axisNo, ctypes.c_double(voltage))
 

    def setFrequency(self, axisNo, frequency):
        '''
        Sets the frequency parameter for an axis

        Parameters
            axisNo	Axis number (0 ... 2)
            frequency	Frequency in Hz, internal resolution is 1 Hz
        Returns
            None
        '''
        ANC.setFrequency(self.device, axisNo, ctypes.c_double(frequency))
        
   
    def setTargetPosition(self, axisNo, target):
        '''
        Sets the target position for automatic motion, see ANC_startAutoMove. For linear type actuators the position unit is m, for goniometers and rotators it is degree.

        Parameters
            axisNo	Axis number (0 ... 2)
            target	Target position [m] or [°]. Internal resulution is 1 nm or 1 µ°.
        Returns
            None
        '''
        ANC.setTargetPosition(self.device, axisNo, ctypes.c_double(target))
        
        
    def setTargetRange(self, axisNo, targetRg):
        '''
        Defines the range around the target position where the target is considered to be reached.

        Parameters
            axisNo	Axis number (0 ... 2)
            targetRg	Target range [m] or [°]. Internal resulution is 1 nm or 1 µ°.
        Returns
            None
        '''
        ANC.setTargetRange(self.device, axisNo, ctypes.c_double(targetRg))
        
        
    def startAutoMove(self, axisNo, enable, relative):
        '''
        Switches automatic moving (i.e. following the target position) on or off

        Parameters
            axisNo	Axis number (0 ... 2)
            enable	Enables (1) or disables (0) automatic motion
            relative	If the target position is to be interpreted absolute (0) or relative to the current position (1)
        Returns
            None
        '''
        ANC.startAutoMove(self.device, axisNo, enable, relative)
        
   
    def startContinuousMove(self, axisNo, start, backward):
        '''
        Starts or stops continous motion in forward direction. Other kinds of motions are stopped.

        Parameters
            axisNo	Axis number (0 ... 2)
            start	Starts (1) or stops (0) the motion
            backward	If the move direction is forward (0) or backward (1)
        Returns
            None
        '''
        ANC.startContinousMove(self.device, axisNo, start, backward)
        
    def startSingleStep(self, axisNo, backward):
        '''
        Triggers a single step in desired direction.

        Parameters
            axisNo	Axis number (0 ... 2)
            backward	If the step direction is forward (0) or backward (1)
        Returns
            None
        '''
        ANC.startSingleStep(self.device, axisNo, backward)



#
#  ANC350lib is a Python implementation of the C++ header provided
#     with the attocube ANC350 closed-loop positioner system.
#
#  It depends on anc350v2.dll which is provided by attocube in the
#     ANC350_DLL folders on the driver disc.
#     This in turn requires nhconnect.dll and libusb0.dll. Place all
#     of these in the same folder as this module (and that of ANC350lib).
#
#                ANC350lib is written by Rob Heath
#                      rob@robheath.me.uk
#                         24-Feb-2015
#                       robheath.me.uk
#
#    v1.1: corrected POINTER used in intEnable (!!)
#


import ctypes
import time
#
# List of error types
#

NCB_Ok = 0 #                    No error
NCB_Error = -1 #                Unknown / other error
NCB_Timeout = 1 #               Timeout during data retrieval
NCB_NotConnected = 2 #          No contact with the positioner via USB
NCB_DriverError = 3 #           Error in the driver response
NCB_BootIgnored	= 4 #           Ignored boot, equipment was already running
NCB_FileNotFound = 5 #          Boot image not found
NCB_InvalidParam = 6 #          Transferred parameter is invalid
NCB_DeviceLocked = 7 #          A connection attempt failed because the device is already in use
NCB_NotSpecifiedParam = 8 #     Transferred parameter is out of specification


#checks the errors returned from the dll
def checkError(code,func,args):
	if code == NCB_Ok:
		return
	elif code == NCB_BootIgnored:	   
		print("Warning: boot ignored in",func.__name__,"with parameters:",args)
		return
	elif code == NCB_Error:			 
		raise Exception("Error: unspecific in"+str(func.__name__)+"with parameters:"+str(args))
	elif code == NCB_Timeout:		   
		raise Exception("Error: comm. timeout in"+str(func.__name__)+"with parameters:"+str(args))
	elif code == NCB_NotConnected:	  
		raise Exception("Error: not connected") 
	elif code == NCB_DriverError:	   
		raise Exception("Error: driver error") 
	elif code == NCB_FileNotFound:	  
		raise Exception("Error: file not found") 
	elif code == NCB_InvalidParam:	  
		raise Exception("Error: invalid parameter")
	elif code == NCB_DeviceLocked:	  
		raise Exception("Error: device locked")
	elif code == NCB_NotSpecifiedParam: 
		raise Exception("Error: unspec. parameter in"+str(func.__name__)+"with parameters:"+str(args))
	else:					
		raise Exception("Error: unknown in"+str(func.__name__)+"with parameters:"+str(args))
	return code

#import dll
anc350v2 = ctypes.windll.anc350v2

#creates alias for c_int as "Int32" (I really don't know why)
Int32 = ctypes.c_int

#structure for PositionerInfo to handle positionerCheck return data
class PositionerInfo(ctypes.Structure):
	_fields_ = [("id",Int32),
				("locked",ctypes.c_bool)]

#aliases for the strangely-named functions from the dll
positionerAcInEnable = getattr(anc350v2,"_PositionerAcInEnable@12")
positionerAmplitude = getattr(anc350v2,"_PositionerAmplitude@12")
positionerAmplitudeControl = getattr(anc350v2,"_PositionerAmplitudeControl@12")
positionerBandwidthLimitEnable = getattr(anc350v2,"_PositionerBandwidthLimitEnable@12")
positionerCapMeasure = getattr(anc350v2,"_PositionerCapMeasure@12")
positionerCheck = getattr(anc350v2,"_PositionerCheck@4")
positionerClearStopDetection = getattr(anc350v2,"_PositionerClearStopDetection@8")
positionerClose = getattr(anc350v2,"_PositionerClose@4")
positionerConnect = getattr(anc350v2,"_PositionerConnect@8")
positionerDcInEnable = getattr(anc350v2,"_PositionerDcInEnable@12")
positionerDCLevel = getattr(anc350v2,"_PositionerDCLevel@12")
positionerDutyCycleEnable = getattr(anc350v2,"_PositionerDutyCycleEnable@8")
positionerDutyCycleOffTime = getattr(anc350v2,"_PositionerDutyCycleOffTime@8")
positionerDutyCyclePeriod = getattr(anc350v2,"_PositionerDutyCyclePeriod@8")
positionerExternalStepBkwInput = getattr(anc350v2,"_PositionerExternalStepBkwInput@12")
positionerExternalStepFwdInput = getattr(anc350v2,"_PositionerExternalStepFwdInput@12")
positionerExternalStepInputEdge = getattr(anc350v2,"_PositionerExternalStepInputEdge@12")
positionerFrequency = getattr(anc350v2,"_PositionerFrequency@12")
positionerGetAcInEnable = getattr(anc350v2,"_PositionerGetAcInEnable@12")
positionerGetAmplitude = getattr(anc350v2,"_PositionerGetAmplitude@12")
positionerGetBandwidthLimitEnable = getattr(anc350v2,"_PositionerGetBandwidthLimitEnable@12")
positionerGetDcInEnable = getattr(anc350v2,"_PositionerGetDcInEnable@12")
positionerGetDcLevel = getattr(anc350v2,"_PositionerGetDcLevel@12")
positionerGetFrequency = getattr(anc350v2,"_PositionerGetFrequency@12")
positionerGetIntEnable = getattr(anc350v2,"_PositionerGetIntEnable@12")
positionerGetPosition = getattr(anc350v2,"_PositionerGetPosition@12")
positionerGetReference = getattr(anc350v2,"_PositionerGetReference@16")
positionerGetReferenceRotCount = getattr(anc350v2,"_PositionerGetReferenceRotCount@12")
positionerGetRotCount = getattr(anc350v2,"_PositionerGetRotCount@12")
positionerGetSpeed = getattr(anc350v2,"_PositionerGetSpeed@12")
positionerGetStatus = getattr(anc350v2,"_PositionerGetStatus@12")
positionerGetStepwidth = getattr(anc350v2,"_PositionerGetStepwidth@12")
positionerIntEnable = getattr(anc350v2,"_PositionerIntEnable@12")
positionerLoad = getattr(anc350v2,"_PositionerLoad@12")
positionerMoveAbsolute = getattr(anc350v2,"_PositionerMoveAbsolute@16")
positionerMoveAbsoluteSync = getattr(anc350v2,"_PositionerMoveAbsoluteSync@8")
positionerMoveContinuous = getattr(anc350v2,"_PositionerMoveContinuous@12")
positionerMoveReference = getattr(anc350v2,"_PositionerMoveReference@8")
positionerMoveRelative = getattr(anc350v2,"_PositionerMoveRelative@16")
positionerMoveSingleStep = getattr(anc350v2,"_PositionerMoveSingleStep@12")
positionerQuadratureAxis = getattr(anc350v2,"_PositionerQuadratureAxis@12")
positionerQuadratureInputPeriod = getattr(anc350v2,"_PositionerQuadratureInputPeriod@12")
positionerQuadratureOutputPeriod = getattr(anc350v2,"_PositionerQuadratureOutputPeriod@12")
positionerResetPosition = getattr(anc350v2,"_PositionerResetPosition@8")
positionerSensorPowerGroupA = getattr(anc350v2,"_PositionerSensorPowerGroupA@8")
positionerSensorPowerGroupB = getattr(anc350v2,"_PositionerSensorPowerGroupB@8")
positionerSetHardwareId = getattr(anc350v2,"_PositionerSetHardwareId@8")
positionerSetOutput = getattr(anc350v2,"_PositionerSetOutput@12")
positionerSetStopDetectionSticky = getattr(anc350v2,"_PositionerSetStopDetectionSticky@12")
positionerSetTargetGround = getattr(anc350v2,"_PositionerSetTargetGround@12")
positionerSetTargetPos = getattr(anc350v2,"_PositionerSetTargetPos@16")
positionerSingleCircleMode = getattr(anc350v2,"_PositionerSingleCircleMode@12")
positionerStaticAmplitude = getattr(anc350v2,"_PositionerStaticAmplitude@8")
positionerStepCount = getattr(anc350v2,"_PositionerStepCount@12")
positionerStopApproach = getattr(anc350v2,"_PositionerStopApproach@8")
positionerStopDetection = getattr(anc350v2,"_PositionerStopDetection@12")
positionerStopMoving = getattr(anc350v2,"_PositionerStopMoving@8")
positionerTrigger = getattr(anc350v2,"_PositionerTrigger@16")
positionerTriggerAxis = getattr(anc350v2,"_PositionerTriggerAxis@12")
positionerTriggerEpsilon = getattr(anc350v2,"_PositionerTriggerEpsilon@12")
positionerTriggerModeIn = getattr(anc350v2,"_PositionerTriggerModeIn@8")
positionerTriggerModeOut = getattr(anc350v2,"_PositionerTriggerModeOut@8")
positionerTriggerPolarity = getattr(anc350v2,"_PositionerTriggerPolarity@12")
positionerUpdateAbsolute = getattr(anc350v2,"_PositionerUpdateAbsolute@12")

#set error checking & handling
positionerAcInEnable.errcheck = checkError
positionerAmplitude.errcheck = checkError
positionerAmplitudeControl.errcheck = checkError
positionerBandwidthLimitEnable.errcheck = checkError
positionerCapMeasure.errcheck = checkError
positionerClearStopDetection.errcheck = checkError
positionerClose.errcheck = checkError
positionerConnect.errcheck = checkError
positionerDcInEnable.errcheck = checkError
positionerDCLevel.errcheck = checkError
positionerDutyCycleEnable.errcheck = checkError
positionerDutyCycleOffTime.errcheck = checkError
positionerDutyCyclePeriod.errcheck = checkError
positionerExternalStepBkwInput.errcheck = checkError
positionerExternalStepFwdInput.errcheck = checkError
positionerExternalStepInputEdge.errcheck = checkError
positionerFrequency.errcheck = checkError
positionerGetAcInEnable.errcheck = checkError
positionerGetAmplitude.errcheck = checkError
positionerGetBandwidthLimitEnable.errcheck = checkError
positionerGetDcInEnable.errcheck = checkError
positionerGetDcLevel.errcheck = checkError
positionerGetFrequency.errcheck = checkError
positionerGetIntEnable.errcheck = checkError
positionerGetPosition.errcheck = checkError
positionerGetReference.errcheck = checkError
positionerGetReferenceRotCount.errcheck = checkError
positionerGetRotCount.errcheck = checkError
positionerGetSpeed.errcheck = checkError
positionerGetStatus.errcheck = checkError
positionerGetStepwidth.errcheck = checkError
positionerIntEnable.errcheck = checkError
positionerLoad.errcheck = checkError
positionerMoveAbsolute.errcheck = checkError
positionerMoveAbsoluteSync.errcheck = checkError
positionerMoveContinuous.errcheck = checkError
positionerMoveReference.errcheck = checkError
positionerMoveRelative.errcheck = checkError
positionerMoveSingleStep.errcheck = checkError
positionerQuadratureAxis.errcheck = checkError
positionerQuadratureInputPeriod.errcheck = checkError
positionerQuadratureOutputPeriod.errcheck = checkError
positionerResetPosition.errcheck = checkError
positionerSensorPowerGroupA.errcheck = checkError
positionerSensorPowerGroupB.errcheck = checkError
positionerSetHardwareId.errcheck = checkError
positionerSetOutput.errcheck = checkError
positionerSetStopDetectionSticky.errcheck = checkError
positionerSetTargetGround.errcheck = checkError
positionerSetTargetPos.errcheck = checkError
positionerSingleCircleMode.errcheck = checkError
positionerStaticAmplitude.errcheck = checkError
positionerStepCount.errcheck = checkError
positionerStopApproach.errcheck = checkError
positionerStopDetection.errcheck = checkError
positionerStopMoving.errcheck = checkError
positionerTrigger.errcheck = checkError
positionerTriggerAxis.errcheck = checkError
positionerTriggerEpsilon.errcheck = checkError
positionerTriggerModeIn.errcheck = checkError
positionerTriggerModeOut.errcheck = checkError
positionerTriggerPolarity.errcheck = checkError
positionerUpdateAbsolute.errcheck = checkError
#positionerCheck.errcheck = checkError
#positionerCheck returns number of attached devices; gives "comms error" if this is applied, despite working fine

#set argtypes
positionerAcInEnable.argtypes = [Int32, Int32, ctypes.c_bool]
positionerAmplitude.argtypes = [Int32, Int32, Int32]
positionerAmplitudeControl.argtypes = [Int32, Int32, Int32]
positionerBandwidthLimitEnable.argtypes = [Int32, Int32, ctypes.c_bool]
positionerCapMeasure.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerCheck.argtypes = [ctypes.POINTER(PositionerInfo)]
positionerClearStopDetection.argtypes = [Int32, Int32]
positionerClose.argtypes = [Int32]
positionerConnect.argtypes = [Int32, ctypes.POINTER(Int32)]
positionerDcInEnable.argtypes = [Int32, Int32, ctypes.c_bool]
positionerDCLevel.argtypes = [Int32, Int32, Int32]
positionerDutyCycleEnable.argtypes = [Int32, ctypes.c_bool]
positionerDutyCycleOffTime.argtypes = [Int32, Int32]
positionerDutyCyclePeriod.argtypes = [Int32, Int32]
positionerExternalStepBkwInput.argtypes = [Int32, Int32, Int32]
positionerExternalStepFwdInput.argtypes = [Int32, Int32, Int32]
positionerExternalStepInputEdge.argtypes = [Int32, Int32, Int32]
positionerFrequency.argtypes = [Int32, Int32, Int32]
positionerGetAcInEnable.argtypes = [Int32, Int32, ctypes.POINTER(ctypes.c_bool)]
positionerGetAmplitude.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerGetBandwidthLimitEnable.argtypes = [Int32, Int32, ctypes.POINTER(ctypes.c_bool)]
positionerGetDcInEnable.argtypes = [Int32, Int32, ctypes.POINTER(ctypes.c_bool)]
positionerGetDcLevel.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerGetFrequency.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerGetIntEnable.argtypes = [Int32, Int32, ctypes.POINTER(ctypes.c_bool)]
positionerGetPosition.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerGetReference.argtypes = [Int32, Int32, ctypes.POINTER(Int32), ctypes.POINTER(ctypes.c_bool)]
positionerGetReferenceRotCount.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerGetRotCount.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerGetSpeed.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerGetStatus.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerGetStepwidth.argtypes = [Int32, Int32, ctypes.POINTER(Int32)]
positionerIntEnable.argtypes = [Int32, Int32, ctypes.c_bool]
positionerLoad.argtypes = [Int32, Int32, ctypes.POINTER(ctypes.c_char)]
positionerMoveAbsolute.argtypes = [Int32, Int32, Int32, Int32]
positionerMoveAbsoluteSync.argtypes = [Int32, Int32]
positionerMoveContinuous.argtypes = [Int32, Int32, Int32]
positionerMoveReference.argtypes = [Int32, Int32]
positionerMoveRelative.argtypes = [Int32, Int32, Int32, Int32]
positionerMoveSingleStep.argtypes = [Int32, Int32, Int32]
positionerQuadratureAxis.argtypes = [Int32, Int32, Int32]
positionerQuadratureInputPeriod.argtypes = [Int32, Int32, Int32]
positionerQuadratureOutputPeriod.argtypes = [Int32, Int32, Int32]
positionerResetPosition.argtypes = [Int32, Int32]
positionerSensorPowerGroupA.argtypes = [Int32, ctypes.c_bool]
positionerSensorPowerGroupB.argtypes = [Int32, ctypes.c_bool]
positionerSetHardwareId.argtypes = [Int32, Int32]
positionerSetOutput.argtypes = [Int32, Int32, ctypes.c_bool]
positionerSetStopDetectionSticky.argtypes = [Int32, Int32, ctypes.c_bool]
positionerSetTargetGround.argtypes = [Int32, Int32, ctypes.c_bool]
positionerSetTargetPos.argtypes = [Int32, Int32, Int32, Int32]
positionerSingleCircleMode.argtypes = [Int32, Int32, ctypes.c_bool]
positionerStaticAmplitude.argtypes = [Int32, Int32]
positionerStepCount.argtypes = [Int32, Int32, Int32]
positionerStopApproach.argtypes = [Int32, Int32]
positionerStopDetection.argtypes = [Int32, Int32, ctypes.c_bool]
positionerStopMoving.argtypes = [Int32, Int32]
positionerTrigger.argtypes = [Int32, Int32, Int32, Int32]
positionerTriggerAxis.argtypes = [Int32, Int32, Int32]
positionerTriggerEpsilon.argtypes = [Int32,Int32, Int32]
positionerTriggerModeIn.argtypes = [Int32, Int32]
positionerTriggerModeOut.argtypes = [Int32, Int32]
positionerTriggerPolarity.argtypes = [Int32, Int32, Int32]
positionerUpdateAbsolute.argtypes = [Int32, Int32, Int32]



#
#  PyANC350 is a control scheme suitable for the Python coding style
#    for the attocube ANC350 closed-loop positioner system.
#
#  It implements ANC350lib, which in turn depends on anc350v2.dll
#    which is provided by attocube in the ANC350_DLL folders
#    on the driver disc.
#    This in turn requires nhconnect.dll and libusb0.dll. Place all
#    of these in the same folder as this module (and that of ANC350lib).
#
#  Unlike ANC350lib which is effectively a re-imagining of the
#    C++ header, PyANC350 is intended to behave as one might expect
#    Python to. This means: returning values; behaving as an object.
#
#  At present this only addresses the first ANC350 connected to the
#    machine.
#
#  Usage:
#  1. instantiate Positioner() class to begin, eg. pos = Positioner().
#  2. methods from the ANC350v2 documentation are implemented such that
#      function PositionerGetPosition(handle, axis, &pos)
#      becomes position = pos.getPosition(axis),
#      PositionerCapMeasure(handle,axis,&cap) becomes
#      cap = pos.capMeasure(axis), and so on. Return code handling is
#      within ANC350lib.
#  3. bitmask() and debitmask() functions have been added for
#      convenience when using certain functions
#      (e.g. getStatus,moveAbsoluteSync)
#  4. for tidiness remember to Positioner.close() when finished!
#
#                PyANC350 is written by Rob Heath
#                      rob@robheath.me.uk
#                         24-Feb-2015
#                       robheath.me.uk

import pyanc350.v2.ANC350lib as ANC350lib
import ctypes
import math

class Positioner:
	def __init__(self):
		self.check()
		self.connect()

	def acInEnable(self, axis, state):
		'''
		Activates/deactivates AC input of addressed axis; only applicable for dither axes
		'''
		ANC350lib.positionerAcInEnable(self.handle,axis,ctypes.c_bool(state))

	def amplitude(self, axis, amp):
		'''
		set the amplitude setpoint in mV
		'''
		ANC350lib.positionerAmplitude(self.handle,axis,amp)

	def amplitudeControl(self, axis, mode):
		'''
		selects the type of amplitude control. The amplitude is controlled by the positioner to hold the value constant determined by the selected type of amplitude control. mode takes values 0: speed, 1: amplitude, 2: step size
		'''
		ANC350lib.positionerAmplitudeControl(self.handle,axis,mode)

	def bandwidthLimitEnable(self, axis, state):
		'''
		activates/deactivates the bandwidth limiter of the addressed axis. only applicable for scanner axes
		'''
		ANC350lib.positionerBandwidthLimitEnable(self.handle,axis,ctypes.c_bool(state))

	def capMeasure(self, axis):
		'''
		determines the capacitance of the piezo addressed by axis
		'''
		self.status = ANC350lib.Int32(0)
		ANC350lib.positionerCapMeasure(self.handle,axis,ctypes.byref(self.status))
		return self.status.value

	def check(self):
		'''
		Determines number of connected positioners and their respective hardware IDs
		'''
		self.posinf = ANC350lib.PositionerInfo() #create PositionerInfo Struct
		self.numconnected = ANC350lib.positionerCheck(ctypes.byref(self.posinf)) #look for positioners!
		if self.numconnected > 0:
			print(self.numconnected,'ANC350 connected')
			print('ANC350 with id:',self.posinf.id,'has locked state:',self.posinf.locked)

	def clearStopDetection(self, axis):
		'''
		when .setStopDetectionSticky() is enabled, this clears the stop detection status
		'''
		ANC350lib.positionerClearStopDetection(self.handle,axis)

	def close(self):
		'''
		closes connection to ANC350 device
		'''
		ANC350lib.positionerClose(self.handle)

	def connect(self):
		'''
		Establishes connection to first device found
		'''
		self.handle = ANC350lib.Int32(0)
		try:
			ANC350lib.positionerConnect(0,ctypes.byref(self.handle)) #0 means "first device"
			print('connected to first positioner')
		except Exception as e:
			print('unable to connect!')
			raise e		

	def dcInEnable(self, axis, state):
		'''
		Activates/deactivates DC input of addressed axis; only applicable for scanner/dither axes
		'''
		ANC350lib.positionerDcInEnable(self.handle,axis,ctypes.c_bool(state))

	def dcLevel(self, axis, dclev):
		'''
		sets the dc level of selected axis. dclevel in mV
		'''
		ANC350lib.positionerDCLevel(self.handle,axis,dclev)

	def dutyCycleEnable(self, state):
		'''
		controls duty cycle mode
		'''
		ANC350lib.positionerDutyCycleEnable(self.handle,ctypes.c_bool(state))

	def dutyCycleOffTime(self, value):
		'''
		sets duty cycle off time
		'''
		ANC350lib.positionerDutyCycleOffTime(self.handle,value)

	def dutyCyclePeriod(self, value):
		'''
		sets duty cycle period
		'''
		ANC350lib.positionerDutyCyclePeriod(self.handle,value)

	def externalStepBkwInput(self, axis, input_trigger):
		'''
		configures external step trigger input for selected axis. a trigger on this input results in a backwards single step. input_trigger: 0 disabled, 1-6 input trigger
		'''
		ANC350lib.positionerExternalStepBkwInput(self.handle,axis,input_trigger)

	def externalStepFwdInput(self, axis, input_trigger):
		'''
		configures external step trigger input for selected axis. a trigger on this input results in a forward single step. input_trigger: 0 disabled, 1-6 input trigger
		'''
		ANC350lib.positionerExternalStepFwdInput(self.handle,axis,input_trigger)

	def externalStepInputEdge(self, axis, edge):
		'''
		configures edge sensitivity of external step trigger input for selected axis. edge: 0 rising, 1 falling
		'''
		ANC350lib.positionerExternalStepInputEdge(self.handle,axis,edge)

	def frequency(self, axis, freq):
		'''
		sets the frequency of selected axis. frequency in Hz
		'''
		ANC350lib.positionerFrequency(self.handle,axis,freq)

	def getAcInEnable(self, axis):
		'''
		determines status of ac input of addressed axis. only applicable for dither axes
		'''
		self.status = ctypes.c_bool(None)
		ANC350lib.positionerGetAcInEnable(self.handle,axis,ctypes.byref(self.status))
		return self.status.value

	def getAmplitude(self, axis):
		'''
		determines the actual amplitude. In case of standstill of the actor this is the amplitude setpoint. In case of movement the amplitude set by amplitude control is determined.
		'''
		self.status = ANC350lib.Int32(0)
		ANC350lib.positionerGetAmplitude(self.handle,axis,ctypes.byref(self.status))
		return self.status.value

	def getBandwidthLimitEnable(self, axis):
		'''
		determines status of bandwidth limiter of addressed axis. only applicable for scanner axes
		'''
		self.status = ctypes.c_bool(None)
		ANC350lib.positionerGetBandwidthLimitEnable(self.handle,axis,ctypes.byref(self.status))
		return self.status.value

	def getDcInEnable(self, axis):
		'''
		determines status of dc input of addressed axis. only applicable for scanner/dither axes
		'''
		self.status = ctypes.c_bool(None)
		ANC350lib.positionerGetDcInEnable(self.handle,axis,ctypes.byref(self.status))
		return self.status.value

	def getDcLevel(self, axis):
		'''
		determines the status actual DC level in mV
		'''
		self.dclev = ANC350lib.Int32(0)
		ANC350lib.positionerGetDcLevel(self.handle,axis,ctypes.byref(self.dclev))
		return self.dclev.value

	def getFrequency(self, axis):
		'''
		determines the frequency in Hz
		'''
		self.freq = ANC350lib.Int32(0)
		ANC350lib.positionerGetFrequency(self.handle,axis,ctypes.byref(self.freq))
		return self.freq.value

	def getIntEnable(self, axis):
		'''
		determines status of internal signal generation of addressed axis. only applicable for scanner/dither axes
		'''
		self.status = ctypes.c_bool(None)
		ANC350lib.positionerGetIntEnable(self.handle,axis,ctypes.byref(self.status))
		return self.status.value

	def getPosition(self, axis):
		'''
		determines actual position of addressed axis
		'''
		self.pos = ANC350lib.Int32(0)
		ANC350lib.positionerGetPosition(self.handle,axis,ctypes.byref(self.pos))
		return self.pos.value

	def getReference(self, axis):
		'''
		determines distance of reference mark to origin
		'''
		self.pos = ANC350lib.Int32(0)
		self.validity = ctypes.c_bool(None)
		ANC350lib.positionerGetReference(self.handle,axis,ctypes.byref(self.pos),ctypes.byref(self.validity))
		return self.pos.value, self.validity.value

	def getReferenceRotCount(self, axis):
		'''
		determines actual position of addressed axis
		'''
		self.rotcount = ANC350lib.Int32(0)
		ANC350lib.positionerGetReferenceRotCount(self.handle,axis,ctypes.byref(self.rotcount))
		return self.rotcount.value

	def getRotCount(self, axis):
		'''
		determines actual number of rotations in case of rotary actuator
		'''
		self.rotcount = ANC350lib.Int32(0)
		ANC350lib.positionerGetRotCount(self.handle,axis,ctypes.byref(self.rotcount))
		return self.rotcount.value

	def getSpeed(self, axis):
		'''
		determines the actual speed. In case of standstill of this actor this is the calculated speed resulting	from amplitude setpoint, frequency, and motor parameters. In case of movement this is measured speed.
		'''
		self.spd = ANC350lib.Int32(0)
		ANC350lib.positionerGetSpeed(self.handle,axis,ctypes.byref(self.spd))
		return self.spd.value

	def getStatus(self, axis):
		'''
		determines the status of the selected axis. result: bit0 (moving), bit1 (stop detected), bit2 (sensor error), bit3 (sensor disconnected)
		'''
		self.status = ANC350lib.Int32(0)
		ANC350lib.positionerGetStatus(self.handle,axis,ctypes.byref(self.status))
		return self.status.value

	def getStepwidth(self, axis):
		'''
		determines the step width. In case of standstill of the motor this is the calculated step width	resulting from amplitude setpoint, frequency, and motor parameters. In case of movement this is measured step width
		'''
		self.stepwdth = ANC350lib.Int32(0)
		ANC350lib.positionerGetStepwidth(self.handle,axis,ctypes.byref(self.stepwdth))
		return self.stepwdth.value

	def intEnable(self, axis, state):
		'''
		Activates/deactivates internal signal generation of addressed axis; only applicable for scanner/dither axes
		'''
		ANC350lib.positionerIntEnable(self.handle,axis,ctypes.c_bool(state))

	def load(self, axis, filename):
		'''
		loads a parameter file for actor configuration.	note: this requires a pointer to a char datatype. having no parameter file to test, I have no way of telling whether this will work, especially with the manual being as erroneous as it is. as such, use at your own (debugging) risk!
		'''
		ANC350lib.positionerLoad(self.handle,axis,ctypes.byref(ctypes.char(filename)))

	def moveAbsolute(self, axis, position, rotcount=0):
		'''
		starts approach to absolute target position. previous movement will be stopped. rotcount optional argument position units are in 'unit of actor multiplied by 1000' (generally nanometres)
		'''
		ANC350lib.positionerMoveAbsolute(self.handle,axis,position,rotcount)

	def moveAbsoluteSync(self, bitmask_of_axes):
		'''
		starts the synchronous approach to absolute target position for selected axis. previous movement will be stopped. target position for each axis defined by .setTargetPos() takes a *bitmask* of axes!
		'''
		ANC350lib.positionerMoveAbsoluteSync(self.handle,bitmask_of_axes)

	def moveContinuous(self, axis, direction):
		'''
		starts continuously positioning with set parameters for ampl and speed and amp control respectively. direction can be 0 (forward) or 1 (backward)
		'''
		ANC350lib.positionerMoveContinuous(self.handle,axis,direction)

	def moveReference(self, axis):
		'''
		starts approach to reference position. previous movement will be stopped.
		'''
		ANC350lib.positionerMoveReference(self.handle,axis)

	def moveRelative(self, axis, position, rotcount=0):
		'''
		starts approach to relative target position. previous movement will be stopped. rotcount optional argument. position units are in 'unit of actor multiplied by 1000' (generally nanometres)
		'''
		ANC350lib.positionerMoveRelative(self.handle,axis,position,rotcount)

	def moveSingleStep(self, axis, direction):
		'''
		starts a one-step positioning. Previous movement will be stopped. direction can be 0 (forward) or 1 (backward)
		'''
		ANC350lib.positionerMoveSingleStep(self.handle,axis,direction)

	def quadratureAxis(self, quadratureno, axis):
		'''
		selects the axis for use with this trigger in/out pair. quadratureno: number of addressed quadrature unit (0-2)
		'''
		ANC350lib.positionerQuadratureAxis(self.handle,quadratureno,axis)

	def quadratureInputPeriod(self, quadratureno, period):
		'''
		selects the stepsize the controller executes when detecting a step on its input AB-signal. quadratureno: number of addressed quadrature unit (0-2). period: stepsize in unit of actor * 1000
		'''
		ANC350lib.positionerQuadratureInputPeriod(self.handle,quadratureno,period)

	def quadratureOutputPeriod(self, quadratureno, period):
		'''
		selects the position difference which causes a step on the output AB-signal. quadratureno: number of addressed quadrature unit (0-2). period: period in unit of actor * 1000
		'''
		ANC350lib.positionerQuadratureOutputPeriod(self.handle,quadratureno,period)

	def resetPosition(self, axis):
		'''
		sets the origin to the actual position
		'''
		ANC350lib.positionerResetPosition(self.handle,axis)

	def sensorPowerGroupA(self, state):
		'''
		switches power of sensor group A. Sensor group A contains either the sensors of axis 1-3 or 1-2 dependent on hardware of controller
		'''
		ANC350lib.positionerSensorPowerGroupA(self.handle,ctypes.c_bool(state))

	def sensorPowerGroupB(self, state):
		'''
		switches power of sensor group B. Sensor group B contains either the sensors of axis 4-6 or 3 dependent on hardware of controller
		'''
		ANC350lib.positionerSensorPowerGroupB(self.handle,ctypes.c_bool(state))

	def setHardwareId(self, hwid):
		'''
		sets the hardware ID for the device (used to differentiate multiple devices)
		'''
		ANC350lib.positionerSetHardwareId(self.handle,hwid)

	def setOutput(self, axis, state):
		'''
		activates/deactivates the addressed axis
		'''
		ANC350lib.positionerSetOutput(self.handle,axis,ctypes.c_bool(state))

	def setStopDetectionSticky(self, axis, state):
		'''
		when enabled, an active stop detection status remains active until cleared manually by .clearStopDetection()
		'''
		ANC350lib.positionerSetStopDetectionSticky(self.handle,axis,state)

	def setTargetGround(self, axis, state):
		'''
		when enabled, actor voltage set to zero after closed-loop positioning finished
		'''
		ANC350lib.positionerSetTargetGround(self.handle,axis,ctypes.c_bool(state))

	def setTargetPos(self, axis, pos, rotcount=0):
		'''
		sets target position for use with .moveAbsoluteSync()
		'''
		ANC350lib.positionerSetTargetPos(self.handle,axis,pos,rotcount)

	def singleCircleMode(self, axis, state):
		'''
		switches single circle mode. In case of activated single circle mode the number of rotations are ignored and the shortest way to target position is used. Only relevant for rotary actors.
		'''
		ANC350lib.positionerSingleCircleMode(self.handle,axis,ctypes.c_bool(state))

	def staticAmplitude(self, amp):
		'''
		sets output voltage for resistive sensors
		'''
		ANC350lib.positionerStaticAmplitude(self.handle,amp)

	def stepCount(self, axis, stps):
		'''
		configures number of successive step scaused by external trigger or manual step request. steps = 1 to 65535
		'''
		ANC350lib.positionerStepCount(self.handle,axis,stps)

	def stopApproach(self, axis):
		'''
		stops approaching target/relative/reference position. DC level of affected axis after stopping depends on setting by .setTargetGround()
		'''
		ANC350lib.positionerStopApproach(self.handle,axis)

	def stopDetection(self, axis, state):
		'''
		switches stop detection on/off
		'''
		ANC350lib.positionerStopDetection(self.handle,axis,ctypes.c_bool(state))

	def stopMoving(self, axis):
		'''
		stops any positioning, DC level of affected axis is set to zero after stopping.
		'''
		ANC350lib.positionerStopMoving(self.handle,axis)

	def trigger(self, triggerno, lowlevel, highlevel):
		'''
		sets the trigger thresholds for the external trigger. triggerno is 0-5, lowlevel/highlevel in units of actor * 1000
		'''
		ANC350lib.positionerTrigger(self.handle,triggerno,lowlevel,highlevel)

	def triggerAxis(self, triggerno, axis):
		'''
		selects the corresponding axis for the addressed trigger. triggerno is 0-5
		'''
		ANC350lib.positionerTriggerAxis(self.handle,triggerno,axis)

	def triggerEpsilon(self, triggerno, epsilon):
		'''
		sets the hysteresis of the external trigger. epsilon in units of actor * 1000
		'''
		ANC350lib.positionerTriggerEpsilon(self.handle,triggerno,epsilon)

	def triggerModeIn(self, mode):
		'''
		selects the mode of the input trigger signals. state: 0 disabled - inputs trigger nothing, 1 quadrature - three pairs of trigger in signals are used to accept AB-signals for relative positioning, 2 coarse - trigger in signals are used to generate coarse steps
		'''
		ANC350lib.positionerTriggerModeIn(self.handle,mode)

	def triggerModeOut(self, mode):
		'''
		selects the mode of the output trigger signals. state: 0 disabled - inputs trigger nothing, 1 position - the trigger outputs reacts to the defined position ranges with the selected polarity, 2 quadrature - three pairs of trigger out signals are used to signal relative movement as AB-signals, 3 IcHaus - the trigger out signals are used to output the internal position signal of num-sensors
		'''
		ANC350lib.positionerTriggerModeOut(self.handle,mode)

	def triggerPolarity(self, triggerno, polarity):
		'''
		sets the polarity of the external trigger, triggerno: 0-5, polarity: 0 low active, 1 high active
		'''
		ANC350lib.positionerTriggerPolarity(self.handle,triggerno,polarity)

	def updateAbsolute(self, axis, position):
		'''
		updates target position for a *running* approach. function has lower performance impact on running approach compared to .moveAbsolute(). position units are in 'unit of actor multiplied by 1000' (generally nanometres)
		'''
		ANC350lib.positionerUpdateAbsolute(self.handle,axis,position)

def bitmask(input_array):
	'''
	takes an array or string and converts to integer bitmask; reads from left to right e.g. 0100 = 2 not 4
	'''
	total = 0
	for i in range(len(input_array)):
		if int(input_array[i]) != 0 and int(input_array[i])!=1:
			raise Exception('nonbinary value in bitmask, panic!')
		else: 
			total += int(input_array[i])*(2**(i))
	return total

def debitmask(input_int,num_bits = False):
	'''
	takes a bitmask and returns a list of which bits are switched; reads from left to right e.g. 2 = [0, 1] not [1, 0]
	'''
	if num_bits == False and input_int>0:
		num_bits = int(math.ceil(math.log(input_int+1,2)))
	elif input_int == 0:
		return [0]

	result_array = [0]*num_bits
	for i in reversed(range(num_bits)):
		if input_int - 2**i >= 0:
			result_array[i] = 1
			input_int -= 2**i
	return result_array



import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name = "pyanc350",
	version = "0.9",
	author = "Robert Heath",
	author_email = "rob@robheath.me.uk",
	description = "Windows wrapper for attocube systems AG's ANC350 control DLL",
	long_description = long_description,
	long_description_content_type = "text/markdown",
	url = "https://github.com/Laukei/attocube-ANC350-Python-library",
	packages = setuptools.find_packages(),
	classifiers = [
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: Microsoft :: Windows",
		"Development Status :: 3 - Alpha"]

	)